@@ -29,6 +29,7 @@
 #include <gsf/gsf-impl-utils.h>
 #include <gsf/gsf-utils.h>
 #include <gsf/gsf-msole-impl.h>
+#include <gsf/gsf-msole-utils.h>
 #include <gsf/gsf-input-proxy.h>
 
 #include <string.h>
@@ -46,7 +47,7 @@ typedef struct {
 
 typedef struct {
 	char	 *name;
-	char	 *collation_name;
+	GsfMSOleSortingKey *key;
 	int	  index;
 	size_t    size;
 	gboolean  use_sb;
@@ -267,13 +268,7 @@ ole_info_get_sb_file (GsfInfileMSOle *parent)
 static gint
 ole_dirent_cmp (MSOleDirent const *a, MSOleDirent const *b)
 {
-	g_return_val_if_fail (a, 0);
-	g_return_val_if_fail (b, 0);
-
-	g_return_val_if_fail (a->collation_name, 0);
-	g_return_val_if_fail (b->collation_name, 0);
-
-	return strcmp (b->collation_name, a->collation_name);
+	return gsf_msole_sorting_key_cmp (a->key, b->key);
 }
 
 /* Parse dirent number @entry and recursively handle its siblings and children.
@@ -361,7 +356,7 @@ ole_dirent_new (GsfInfileMSOle *ole, guint32 entry, MSOleDirent *parent,
 	/* be really anal in the face of screwups */
 	if (dirent->name == NULL)
 		dirent->name = g_strdup ("");
-	dirent->collation_name = g_utf8_collate_key (dirent->name, -1);
+	dirent->key = gsf_msole_sorting_key_new (dirent->name);
 
 #if 0
 	printf ("%c '%s' :\tsize = %d\tfirst_block = 0x%x\n",
@@ -374,8 +369,8 @@ ole_dirent_new (GsfInfileMSOle *ole, guint32 entry, MSOleDirent *parent,
 			dirent, (GCompareFunc)ole_dirent_cmp);
 
 	/* NOTE : These links are a tree, not a linked list */
-	ole_dirent_new (ole, prev, parent, seen_before); 
-	ole_dirent_new (ole, next, parent, seen_before); 
+	ole_dirent_new (ole, prev, parent, seen_before);
+	ole_dirent_new (ole, next, parent, seen_before);
 
 	if (dirent->is_directory)
 		ole_dirent_new (ole, child, dirent, seen_before);
@@ -392,7 +387,7 @@ ole_dirent_free (MSOleDirent *dirent)
 	g_return_if_fail (dirent != NULL);
 
 	g_free (dirent->name);
-	g_free (dirent->collation_name);
+	gsf_msole_sorting_key_free (dirent->key);
 
 	for (tmp = dirent->children; tmp; tmp = tmp->next)
 		ole_dirent_free ((MSOleDirent *)tmp->data);
@@ -415,7 +410,7 @@ ole_info_unref (MSOleInfo *info)
 		info->root_dir = NULL;
 	}
 	if (info->sb_file != NULL)  {
-		g_object_unref (G_OBJECT (info->sb_file));
+		g_object_unref (info->sb_file);
 		info->sb_file = NULL;
 	}
 	g_free (info);
@@ -626,7 +621,7 @@ gsf_infile_msole_finalize (GObject *obj)
 	GsfInfileMSOle *ole = GSF_INFILE_MSOLE (obj);
 
 	if (ole->input != NULL) {
-		g_object_unref (G_OBJECT (ole->input));
+		g_object_unref (ole->input);
 		ole->input = NULL;
 	}
 	if (ole->info != NULL &&
@@ -734,7 +729,7 @@ gsf_infile_msole_seek (GsfInput *input, gsf_off_t offset, GSeekType whence)
 {
 	GsfInfileMSOle *ole = GSF_INFILE_MSOLE (input);
 
-	(void) offset; 
+	(void) offset;
 	(void) whence;
 
 	ole->cur_block = BAT_MAGIC_UNUSED;
@@ -782,7 +777,7 @@ gsf_infile_msole_new_child (GsfInfileMSOle *parent,
 			if (err != NULL)
 				*err = g_error_new (gsf_input_error_id (), 0,
 						    "Failed to access child");
-			g_object_unref (G_OBJECT (child));
+			g_object_unref (child);
 			return NULL;
 		}
 	} else {
@@ -790,7 +785,7 @@ gsf_infile_msole_new_child (GsfInfileMSOle *parent,
 		size_guess = dirent->size >> info->bb.shift;
 	}
 	if (ole_make_bat (metabat, size_guess + 1, dirent->first_block, &child->bat)) {
-		g_object_unref (G_OBJECT (child));
+		g_object_unref (child);
 		return NULL;
 	}
 
@@ -813,14 +808,14 @@ gsf_infile_msole_new_child (GsfInfileMSOle *parent,
 
 				g_warning ("failure reading block %d for '%s'", i, dirent->name);
 				if (err) *err = g_error_new (gsf_input_error_id (), 0, "failure reading block");
-				g_object_unref (G_OBJECT (child));
+				g_object_unref (child);
 				return NULL;
 			}
 
 		if (remaining > 0) {
 			if (err) *err = g_error_new (gsf_input_error_id (), 0, "insufficient blocks");
 			g_warning ("Small-block file '%s' has insufficient blocks (%u) for the stated size (%lu)", dirent->name, child->bat.num_blocks, (long)dirent->size);
-			g_object_unref (G_OBJECT (child));
+			g_object_unref (child);
 			return NULL;
 		}
 	}
@@ -890,7 +885,7 @@ gsf_infile_msole_init (GObject *obj)
 	ole->bat.num_blocks	= 0;
 	ole->cur_block		= BAT_MAGIC_UNUSED;
 	ole->stream.buf		= NULL;
-	ole->stream.buf_size	= 0; 
+	ole->stream.buf_size	= 0;
 }
 
 static void
@@ -944,7 +939,7 @@ gsf_infile_msole_new (GsfInput *source, GError **err)
 		/* We do this so other kinds of archives can be tried.  */
 		(void)gsf_input_seek (source, calling_pos, G_SEEK_SET);
 
-		g_object_unref (G_OBJECT (ole));
+		g_object_unref (ole);
 		return NULL;
 	}
 
