@@ -3,19 +3,19 @@
  *
  * Copyright (C) 2008 Morten Welinder (terra@gnome.org)
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2.1 of the GNU Lesser General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
+ * USA
  *
  * TODO:
  *   symlinks
@@ -283,7 +283,7 @@ tar_init_info (GsfInfileTar *tar)
 
 		/* Round up to block size */
 		length = (length + (BLOCK_SIZE - 1)) / BLOCK_SIZE * BLOCK_SIZE;
-		
+
 		if (!tar->err &&
 		    gsf_input_seek (tar->source, offset + length, G_SEEK_SET)) {
 			tar->err = g_error_new (gsf_input_error_id (), 0,
@@ -419,28 +419,28 @@ static void
 gsf_infile_tar_finalize (GObject *obj)
 {
 	GsfInfileTar *tar = GSF_INFILE_TAR (obj);
+	g_array_free (tar->children, TRUE);
+	parent_class->finalize (obj);
+}
 
-	if (tar->source != NULL) {
-		g_object_unref (G_OBJECT (tar->source));
-		tar->source = NULL;
-	}
-
-	if (tar->children) {
-		unsigned ui;
-		for (ui = 0; ui < tar->children->len; ui++) {
-			TarChild *c = &g_array_index (tar->children,
-						      TarChild,
-						      ui);
-			g_free (c->name);
-			if (c->dir)
-				g_object_unref (c->dir);
-		}
-		g_array_free (tar->children, TRUE);
-	}
+static void
+gsf_infile_tar_dispose (GObject *obj)
+{
+	GsfInfileTar *tar = GSF_INFILE_TAR (obj);
+	unsigned ui;
 
+	gsf_infile_tar_set_source (tar, NULL);
 	g_clear_error (&tar->err);
 
-	parent_class->finalize (obj);
+	for (ui = 0; ui < tar->children->len; ui++) {
+		TarChild *c = &g_array_index (tar->children, TarChild, ui);
+		g_free (c->name);
+		if (c->dir)
+			g_object_unref (c->dir);
+	}
+	g_array_set_size (tar->children, 0);
+
+	parent_class->dispose (obj);
 }
 
 static GObject*
@@ -523,6 +523,7 @@ gsf_infile_tar_class_init (GObjectClass *gobject_class)
 
 	gobject_class->constructor      = gsf_infile_tar_constructor;
 	gobject_class->finalize		= gsf_infile_tar_finalize;
+	gobject_class->dispose		= gsf_infile_tar_dispose;
 	gobject_class->get_property     = gsf_infile_tar_get_property;
 	gobject_class->set_property     = gsf_infile_tar_set_property;
 
