@@ -29,7 +29,9 @@
 #include <gsf/gsf-doc-meta-data.h>
 #include <gsf/gsf-timestamp.h>
 #include <gsf/gsf-docprop-vector.h>
+#include <gsf/gsf-impl-utils.h>
 #include <string.h>
+#include <glib/gi18n-lib.h>
 
 
 #define OFFICE	 "office:"
@@ -45,13 +47,13 @@ typedef struct {
 G_MODULE_EXPORT char const *
 get_gsf_odf_version_string (void)
 {
-	return "1.1";
+	return "1.2";
 }
 
-G_MODULE_EXPORT short 
+G_MODULE_EXPORT short
 get_gsf_odf_version (void)
 {
-	return 101;
+	return 102;
 }
 
 
@@ -131,7 +133,7 @@ GsfXMLInNS gsf_ooo_ns[] = {
 	GSF_XML_IN_NS (OO_LOTUS_NS_PRODTOOLS, "http://www.ibm.com/xmlns/prodtools"),
 
 	/* CleverAge ODF Add-in for Microsoft Office 3.0.5224.0 (11.0.8302)*/
-	GSF_XML_IN_NS (OO_CLEVERAGE_NS_DC,	"http://purl.org/dc/terms/"), 
+	GSF_XML_IN_NS (OO_CLEVERAGE_NS_DC,	"http://purl.org/dc/terms/"),
 
 	/* KOffice 1.6.3 */
 	GSF_XML_IN_NS (OO_KDE_NS_KOFFICE, "http://www.koffice.org/2005/"),
@@ -211,11 +213,19 @@ static void
 od_meta_user_defined (GsfXMLIn *xin,  xmlChar const **attrs)
 {
 	GsfOOMetaIn *mi = (GsfOOMetaIn *)xin->user_state;
+	mi->typ = G_TYPE_NONE;
+	mi->name = NULL;
 
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
 		if (!strcmp (CXML2C (attrs[0]), "meta:name"))
 			mi->name = g_strdup (CXML2C (attrs[1]));
-		else if (!strcmp (CXML2C (attrs[0]), "meta:type")) {
+		else if (!strcmp (CXML2C (attrs[0]), "meta:value-type") ||
+			 !strcmp (CXML2C (attrs[0]), "meta:type")) {
+				/*
+				 * "meta:type" is a typo on the write
+				 * side that was
+				 * fixed on 20110509.
+				 */
 			if (!strcmp (CXML2C (attrs[1]), "boolean")) {
 				mi->typ = G_TYPE_BOOLEAN;
 			} else if (!strcmp (CXML2C (attrs[1]), "float")) {
@@ -229,30 +239,71 @@ od_meta_user_defined (GsfXMLIn *xin,  xmlChar const **attrs)
 				 * fixed on 20110311.
 				 */
 				mi->typ = GSF_TIMESTAMP_TYPE;
+			} else if (!strcmp (CXML2C (attrs[1]), "time")) {
+				mi->typ = G_TYPE_STRING;
+				/* We should be able to do better */
 			} else {
 				/* What? */
 			}
 		}
 	}
+	if (mi->name == NULL) /* This should not happen */
+		mi->name = g_strdup ("");
 }
 
 static void
 od_meta_user_defined_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
 	GsfOOMetaIn *mi = (GsfOOMetaIn *)xin->user_state;
-	
+
 	if (mi->name != NULL) {
 		GValue *res = g_new0 (GValue, 1);
 		GType t = mi->typ;
 		if (t == G_TYPE_NONE) t = G_TYPE_STRING;
 		if (gsf_xml_gvalue_from_str (res, t, xin->content->str)) {
+			if (g_str_has_prefix (mi->name, "GSF_DOCPROP_VECTOR:")) {
+				gchar *true_name = strchr (mi->name + 19, ':');
+				if (true_name != NULL && *(++true_name) != 0) {
+					GsfDocProp *prop = gsf_doc_meta_data_lookup 
+						(mi->md, true_name);
+					if (prop == NULL) {
+						GsfDocPropVector *vector = gsf_docprop_vector_new ();
+						GValue *val_vector = g_new0 (GValue, 1);
+						g_value_init (val_vector, GSF_DOCPROP_VECTOR_TYPE);
+						gsf_docprop_vector_append (vector, res);
+						g_value_set_object (val_vector, vector);
+						gsf_doc_meta_data_insert (mi->md, g_strdup (true_name),
+									  val_vector);
+						g_value_unset (res);
+						g_free (res);
+					} else {
+						GValue const *old = gsf_doc_prop_get_val (prop);
+						if (G_VALUE_HOLDS (old, GSF_DOCPROP_VECTOR_TYPE)) {
+							GValue *new = g_new0 (GValue, 1);
+							GValueArray *vector;
+							g_value_init (new, GSF_DOCPROP_VECTOR_TYPE);
+							g_value_copy (old, new);
+							vector = gsf_value_get_docprop_varray (new);
+							g_value_array_append (vector, res);
+							gsf_doc_prop_set_val (prop, new);
+						} else 
+							g_printerr (_("Property \"%s\" used for "
+								      "multiple types!"), true_name);
+						g_value_unset (res);
+						g_free (res);						
+					}
+				
+					g_free (mi->name);
+					mi->name = NULL;
+					return;
+				}
+			}
 			gsf_doc_meta_data_insert (mi->md, mi->name, res);
-			mi->name = NULL;
 		} else {
 			g_free (res);
 			g_free (mi->name);
-			mi->name = NULL;
 		}
+		mi->name = NULL;
 	}
 }
 
@@ -290,7 +341,7 @@ static GsfXMLInNode const gsf_opendoc_meta_st_dtd[] = {
 };
 
 
-static void 
+static void
 gsf_opendoc_metadata_subtree_free (G_GNUC_UNUSED GsfXMLIn *xin, gpointer old_state)
 {
 	GsfOOMetaIn *state = old_state;
@@ -304,7 +355,7 @@ gsf_opendoc_metadata_subtree_free (G_GNUC_UNUSED GsfXMLIn *xin, gpointer old_sta
 		g_object_unref (state->keywords);
 	}
 
-	g_object_unref (G_OBJECT (state->md));
+	g_object_unref (state->md);
 	g_free (state);
 }
 
@@ -330,7 +381,7 @@ gsf_opendoc_metadata_subtree (GsfXMLIn *xin, GsfDocMetaData *md)
 	state = g_new0 (GsfOOMetaIn, 1);
 	state->md = md;
 	state->typ = G_TYPE_NONE;
-	g_object_ref (G_OBJECT (md));
+	g_object_ref (md);
 	gsf_xml_in_push_state (xin, doc_subtree, state, gsf_opendoc_metadata_subtree_free, NULL);
 }
 
@@ -400,7 +451,7 @@ od_map_prop_name (char const *name)
 	/* shared by all instances and never freed */
 	static GHashTable *od_prop_name_map = NULL;
 
-	if (NULL == od_prop_name_map) 
+	if (NULL == od_prop_name_map)
 	{
 		static struct {
 			char const *gsf_key;
@@ -466,6 +517,53 @@ meta:object-count
 	GSF_META_NAME_OBJECT_COUNT:
 #endif
 
+static void
+meta_write_props_user_defined (char const *prop_name, GValue const *val, GsfXMLOut *output)
+{
+	GType t;
+	char const *type_name = NULL;
+	
+	gsf_xml_out_start_element (output, "meta:user-defined");
+	gsf_xml_out_add_cstr (output, "meta:name", prop_name);
+	
+	if (NULL == val) {
+			gsf_xml_out_end_element (output);
+			return;
+	}
+	
+	t = G_VALUE_TYPE (val);
+	switch (t) {
+	case G_TYPE_CHAR:
+	case G_TYPE_UCHAR:
+	case G_TYPE_STRING:
+	case G_TYPE_ENUM:
+	case G_TYPE_FLAGS:
+		type_name = "string";
+		break;
+	case G_TYPE_BOOLEAN:
+		type_name = "boolean";
+		break;
+	case G_TYPE_INT:
+	case G_TYPE_UINT:
+	case G_TYPE_LONG:
+	case G_TYPE_ULONG:
+	case G_TYPE_FLOAT:
+	case G_TYPE_DOUBLE:
+		type_name = "float";
+		break;
+		
+	default:
+		if (GSF_TIMESTAMP_TYPE == t)
+			type_name = "date";
+	}
+	if (NULL != type_name)
+		gsf_xml_out_add_cstr (output, "meta:value-type", type_name);
+	if (NULL != val)
+		gsf_xml_out_add_gvalue (output, NULL, val);
+	gsf_xml_out_end_element (output);
+}
+
+
 static void
 meta_write_props (char const *prop_name, GsfDocProp *prop, GsfXMLOut *output)
 {
@@ -501,75 +599,172 @@ meta_write_props (char const *prop_name, GsfDocProp *prop, GsfXMLOut *output)
 	}
 
 	if (NULL == (mapped_name = od_map_prop_name (prop_name))) {
-		GType t;
-		char const *type_name = NULL;
+		if (GSF_DOCPROP_VECTOR_TYPE == G_VALUE_TYPE (val)) {
+			GValueArray	 *vector = gsf_value_get_docprop_varray	(val);
+			guint		  i;
+			guint		  num_values = vector->n_values;
 
-		gsf_xml_out_start_element (output, "meta:user-defined");
-		gsf_xml_out_add_cstr (output, "meta:name", prop_name);
+			for (i = 0; i < num_values; i++) {
+				GValue	*v;
+				char    *new_name = g_strdup_printf 
+					("GSF_DOCPROP_VECTOR:%.4i:%s", i, prop_name);
+				
+				v = g_value_array_get_nth (vector, i);
+				meta_write_props_user_defined (new_name, v, output);
+				g_free (new_name);
+			}
+		} else
+			meta_write_props_user_defined (prop_name, val, output);
+		return;
+	}
 
-		if (NULL == val) {
-			gsf_xml_out_end_element (output);
-			return;
-		}
-
-		t = G_VALUE_TYPE (val);
-		switch (t) {
-		case G_TYPE_CHAR:
-		case G_TYPE_UCHAR:
-		case G_TYPE_STRING:
-		case G_TYPE_ENUM:
-		case G_TYPE_FLAGS:
-			type_name = "string";
-			break;
-		case G_TYPE_BOOLEAN:
-			type_name = "boolean";
-			break;
-		case G_TYPE_INT:
-		case G_TYPE_UINT:
-		case G_TYPE_LONG:
-		case G_TYPE_ULONG:
-		case G_TYPE_FLOAT:
-		case G_TYPE_DOUBLE:
-			type_name = "float";
-			break; 
-
-		default:
-			if (GSF_TIMESTAMP_TYPE == t)
-				type_name = "date";
-		}
-		if (NULL != type_name)
-			gsf_xml_out_add_cstr (output, "meta:type", type_name);
-	} else
-		gsf_xml_out_start_element (output, mapped_name);
+	/* Standardized  ODF meta items*/
+	gsf_xml_out_start_element (output, mapped_name);
 	if (NULL != val)
 		gsf_xml_out_add_gvalue (output, NULL, val);
 	gsf_xml_out_end_element (output);
 }
 
 gboolean
-gsf_opendoc_metadata_write (GsfXMLOut *output, GsfDocMetaData const *md)
+gsf_opendoc_metadata_write (gpointer output, GsfDocMetaData const *md)
 {
+	char *ver_str;
+	GsfXMLOut *xout;
+	GsfODFOut *oout;
+
 	if (output == NULL)
 		return FALSE;
 
-	gsf_xml_out_start_element (output, OFFICE "document-meta");
-	gsf_xml_out_add_cstr_unchecked (output, "xmlns:office",
-		"urn:oasis:names:tc:opendocument:xmlns:office:1.0");
-	gsf_xml_out_add_cstr_unchecked (output, "xmlns:xlink",
-		"http://www.w3.org/1999/xlink");
-	gsf_xml_out_add_cstr_unchecked (output, "xmlns:dc",
-		"http://purl.org/dc/elements/1.1/");
-	gsf_xml_out_add_cstr_unchecked (output, "xmlns:meta",
-		"urn:oasis:names:tc:opendocument:xmlns:meta:1.0");
-	gsf_xml_out_add_cstr_unchecked (output, "xmlns:ooo",
-		"http://openoffice.org/2004/office");
-	gsf_xml_out_add_cstr_unchecked (output, "office:version", 
-					get_gsf_odf_version_string ());
-	gsf_xml_out_start_element (output, OFFICE "meta");
-	gsf_doc_meta_data_foreach (md, (GHFunc) meta_write_props, output);
-	gsf_xml_out_end_element (output); /* </office:meta> */
+	/* For compatibility we take a GsfXMLOut argument.  It really
+	   ought to be a GsfODFOut.  */
+	xout = GSF_XML_OUT (output);
+	oout = GSF_IS_ODF_OUT (xout) ? GSF_ODF_OUT (xout) : NULL;
 
-	gsf_xml_out_end_element (output); /* </office:document-meta> */
+	ver_str = oout
+		? gsf_odf_out_get_version_string (oout)
+		: g_strdup (get_gsf_odf_version_string ());
+
+	gsf_xml_out_start_element (xout, OFFICE "document-meta");
+	gsf_xml_out_add_cstr_unchecked (xout, "xmlns:office",
+		"urn:oasis:names:tc:opendocument:xmlns:office:1.0");
+	gsf_xml_out_add_cstr_unchecked (xout, "xmlns:xlink",
+		"http://www.w3.org/1999/xlink");
+	gsf_xml_out_add_cstr_unchecked (xout, "xmlns:dc",
+		"http://purl.org/dc/elements/1.1/");
+	gsf_xml_out_add_cstr_unchecked (xout, "xmlns:meta",
+		"urn:oasis:names:tc:opendocument:xmlns:meta:1.0");
+	gsf_xml_out_add_cstr_unchecked (xout, "xmlns:ooo",
+		"http://openoffice.org/2004/office");
+	gsf_xml_out_add_cstr_unchecked (xout, "office:version", ver_str);
+	gsf_xml_out_start_element (xout, OFFICE "meta");
+	gsf_doc_meta_data_foreach (md, (GHFunc) meta_write_props, xout);
+	gsf_xml_out_end_element (xout); /* </office:meta> */
+
+	gsf_xml_out_end_element (xout); /* </office:document-meta> */
+
+	g_free (ver_str);
 
 	return TRUE;
 }
+
+/****************************************************************************/
+
+typedef struct _GsfODFOutPrivate {
+	int odf_version;
+} GsfODFOutPrivate;
+
+enum {
+	PROP_0,
+	PROP_ODF_VERSION
+};
+
+static GObjectClass *parent_class;
+
+static void
+gsf_odf_out_set_property (GObject      *object,
+			  guint         property_id,
+			  GValue const *value,
+			  GParamSpec   *pspec)
+{
+	GsfODFOut *xout = (GsfODFOut *)object;
+	GsfODFOutPrivate *priv = xout->priv;
+
+	switch (property_id) {
+	case PROP_ODF_VERSION:
+		priv->odf_version = g_value_get_int (value);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
+static void
+gsf_odf_out_get_property (GObject     *object,
+			  guint        property_id,
+			  GValue      *value,
+			  GParamSpec  *pspec)
+{
+	GsfODFOut const *xout = (GsfODFOut const *)object;
+	GsfODFOutPrivate const *priv = xout->priv;
+
+	switch (property_id) {
+	case PROP_ODF_VERSION:
+		g_value_set_int (value, priv->odf_version);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
+static void
+gsf_odf_out_init (GObject *obj)
+{
+	GsfODFOut *xout = GSF_ODF_OUT (obj);
+	GsfODFOutPrivate *priv = G_TYPE_INSTANCE_GET_PRIVATE
+		(obj, GSF_ODF_OUT_TYPE, GsfODFOutPrivate);
+	xout->priv = priv;
+	priv->odf_version = 100;
+}
+
+static void
+gsf_odf_out_class_init (GObjectClass *gobject_class)
+{
+	parent_class = g_type_class_peek_parent (gobject_class);
+
+	gobject_class->get_property = gsf_odf_out_get_property;
+	gobject_class->set_property = gsf_odf_out_set_property;
+
+	g_object_class_install_property
+		(gobject_class, PROP_ODF_VERSION,
+		 g_param_spec_int ("odf-version",
+				   _("ODF version"),
+				   _("The ODF version this object is targeting as an integer like 100"),
+				   0,
+				   G_MAXINT,
+				   100,
+				   GSF_PARAM_STATIC |
+				   G_PARAM_READWRITE | 
+				   G_PARAM_CONSTRUCT_ONLY));
+
+	g_type_class_add_private (gobject_class, sizeof (GsfODFOutPrivate));
+}
+
+GSF_CLASS (GsfODFOut, gsf_odf_out,
+	   gsf_odf_out_class_init, gsf_odf_out_init,
+	   GSF_XML_OUT_TYPE)
+
+int
+gsf_odf_out_get_version (GsfODFOut *oout)
+{
+	g_return_val_if_fail (GSF_IS_ODF_OUT (oout), 100);
+	return oout->priv->odf_version;
+}
+
+char *
+gsf_odf_out_get_version_string (GsfODFOut *oout)
+{
+	int ver = gsf_odf_out_get_version (oout);
+	return g_strdup_printf ("%d.%d", ver / 100, ver % 100);
+}
