@@ -1,6 +1,6 @@
 /* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * gsf-outfile-msole.c: 
+ * gsf-outfile-msole.c:
  *
  * Copyright (C) 2002-2006 Jody Goldberg (jody@gnome.org)
  *
@@ -24,6 +24,7 @@
 #include <gsf/gsf-outfile-msole.h>
 #include <gsf/gsf-impl-utils.h>
 #include <gsf/gsf-msole-impl.h>
+#include <gsf/gsf-msole-utils.h>
 #include <gsf/gsf-utils.h>
 
 #include <string.h>
@@ -48,6 +49,8 @@ struct _GsfOutfileMSOle {
 	GsfOutput    	*sink;
 	GsfOutfileMSOle	*root;
 
+	GsfMSOleSortingKey *key;
+
 	MSOleOutfileType type;
 	unsigned	 first_block;
 	unsigned	 blocks;
@@ -80,11 +83,14 @@ gsf_outfile_msole_finalize (GObject *obj)
 	GsfOutfileMSOle *ole = GSF_OUTFILE_MSOLE (obj);
 	GsfOutput *output = GSF_OUTPUT (obj);
 
+	gsf_msole_sorting_key_free (ole->key);
+	ole->key = NULL;
+
 	if (!gsf_output_is_closed (output))
 		gsf_output_close (output);
 
 	if (ole->sink != NULL) {
-		g_object_unref (G_OBJECT (ole->sink));
+		g_object_unref (ole->sink);
 		ole->sink = NULL;
 	}
 	switch (ole->type) {
@@ -105,6 +111,7 @@ gsf_outfile_msole_finalize (GObject *obj)
 	default :
 		g_warning ("Unknown file type");
 	}
+
 	parent_class->finalize (obj);
 }
 
@@ -218,7 +225,7 @@ ole_write_const (GsfOutput *sink, guint32 value, unsigned n)
 static void
 ole_pad_bat_unused (GsfOutfileMSOle *ole, unsigned residual)
 {
-	ole_write_const (ole->sink, BAT_MAGIC_UNUSED, 
+	ole_write_const (ole->sink, BAT_MAGIC_UNUSED,
 		(ole_bytes_left_in_block (ole) / BAT_INDEX_SIZE) - residual);
 }
 
@@ -456,7 +463,7 @@ recalc_bat_bat :
 
 	/* free the children */
 	for (i = 0 ; i < elem->len ; i++)
-		g_object_unref (G_OBJECT (g_ptr_array_index (elem, i)));
+		g_object_unref (g_ptr_array_index (elem, i));
 	g_ptr_array_free (elem, TRUE);
 	ole->content.dir.root_order = NULL;
 
@@ -513,7 +520,7 @@ gsf_outfile_msole_write (GsfOutput *output,
 			g_warning ("File too big");
 			return FALSE;
 		}
-		
+
 		ole->first_block = ole_cur_block (ole);
 		ole->type = MSOLE_BIG_BLOCK;
 		wsize = output->cur_size;
@@ -557,7 +564,7 @@ static void
 ole_register_child (GsfOutfileMSOle *root, GsfOutfileMSOle *child)
 {
 	child->root = root;
-	g_object_ref (G_OBJECT (child));
+	g_object_ref (child);
 	child->child_index = root->content.dir.root_order->len;
 	g_ptr_array_add (root->content.dir.root_order, child);
 }
@@ -565,23 +572,17 @@ ole_register_child (GsfOutfileMSOle *root, GsfOutfileMSOle *child)
 static gint
 ole_name_cmp (GsfOutfileMSOle const *a, GsfOutfileMSOle const *b)
 {
-	/* According to the docs length is more important than lexical order */
-	char const *a_name = gsf_output_name ((GsfOutput const *)a);
-	char const *b_name = gsf_output_name ((GsfOutput const *)b);
+	return gsf_msole_sorting_key_cmp (a->key, b->key);
+}
 
-	/* be anal */
-	if (a_name == NULL)
-		return (b_name == NULL) ? 0 : -1;
-	else if (b_name == NULL)
-		return 1;
-	else {
-		unsigned a_len = g_utf8_strlen (a_name, -1);
-		unsigned b_len = g_utf8_strlen (b_name, -1);
-
-		if (a_len != b_len)
-			return a_len - b_len;
-		return g_utf8_collate (a_name, b_name);
-	}
+static void
+make_sorting_name (GsfOutfileMSOle *ole,
+		   G_GNUC_UNUSED GParamSpec *pspec,
+		   G_GNUC_UNUSED gpointer user)
+{
+	const char *name = gsf_output_name (GSF_OUTPUT (ole));
+	gsf_msole_sorting_key_free (ole->key);
+	ole->key = gsf_msole_sorting_key_new (name);
 }
 
 static void
@@ -615,11 +616,11 @@ gsf_outfile_msole_new_child (GsfOutfile *parent,
 		child->type = MSOLE_SMALL_BLOCK;
 		child->content.small_block.buf = g_new0 (guint8, OLE_DEFAULT_THRESHOLD);
 	}
-	g_object_ref (G_OBJECT (ole_parent->sink));
+	g_object_ref (ole_parent->sink);
 	child->sink   = ole_parent->sink;
 	child->root   = ole_parent->root;
 	gsf_outfile_msole_set_block_shift (child,
-		ole_parent->bb.shift, ole_parent->sb.shift); 
+		ole_parent->bb.shift, ole_parent->sb.shift);
 	gsf_output_set_name (GSF_OUTPUT (child), name);
 	gsf_output_set_container (GSF_OUTPUT (child), parent);
 
@@ -640,7 +641,7 @@ gsf_outfile_msole_init (GObject *obj)
 	ole->root   = NULL;
 	ole->type   = MSOLE_DIR;
 
-	gsf_outfile_msole_set_block_shift (ole, 
+	gsf_outfile_msole_set_block_shift (ole,
 		OLE_DEFAULT_BB_SHIFT, OLE_DEFAULT_SB_SHIFT);
 
 	ole->content.dir.children = NULL;
@@ -648,6 +649,22 @@ gsf_outfile_msole_init (GObject *obj)
 	memset (ole->clsid, 0, sizeof (ole->clsid));
 }
 
+static GObject*
+gsf_outfile_msole_constructor (GType                  type,
+			       guint                  n_construct_properties,
+			       GObjectConstructParam *construct_params)
+{
+	GObject *obj = parent_class->constructor (type,
+						  n_construct_properties,
+						  construct_params);
+	GsfOutfileMSOle *ole = (GsfOutfileMSOle *)obj;
+	g_signal_connect (obj,
+			  "notify::name",
+			  G_CALLBACK (make_sorting_name), NULL);
+	make_sorting_name (ole, NULL, NULL);
+	return obj;
+}
+
 static void
 gsf_outfile_msole_class_init (GObjectClass *gobject_class)
 {
@@ -655,6 +672,7 @@ gsf_outfile_msole_class_init (GObjectClass *gobject_class)
 	GsfOutfileClass *outfile_class = GSF_OUTFILE_CLASS (gobject_class);
 
 	gobject_class->finalize		= gsf_outfile_msole_finalize;
+	gobject_class->constructor	= gsf_outfile_msole_constructor;
 	output_class->Close		= gsf_outfile_msole_close;
 	output_class->Seek		= gsf_outfile_msole_seek;
 	output_class->Write		= gsf_outfile_msole_write;
@@ -712,7 +730,7 @@ gsf_outfile_msole_new_full (GsfOutput *sink, guint bb_size, guint sb_size)
 	ole = g_object_new (GSF_OUTFILE_MSOLE_TYPE, NULL);
 	if (G_UNLIKELY (NULL == ole)) return NULL;
 
-	g_object_ref (G_OBJECT (sink));
+	g_object_ref (sink);
 	ole->sink = sink;
 	ole->type = MSOLE_DIR;
 	ole->content.dir.root_order = g_ptr_array_new ();
@@ -771,7 +789,7 @@ gsf_outfile_msole_new_full (GsfOutput *sink, guint bb_size, guint sb_size)
 GsfOutfile *
 gsf_outfile_msole_new (GsfOutput *sink)
 {
-	return gsf_outfile_msole_new_full (sink, 
+	return gsf_outfile_msole_new_full (sink,
 		OLE_DEFAULT_BB_SIZE, OLE_DEFAULT_SB_SIZE);
 }
 
