@@ -38,8 +38,11 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #endif
 
-#ifdef HAVE_SETRLIMI
+#ifdef HAVE_SETRLIMIT
 #include <sys/resource.h>
+#if !defined(RLIMIT_AS)
+#define RLIMIT_AS	RLIMIT_DATA
+#endif
 #endif
 
 G_GNUC_NORETURN static void
@@ -147,7 +150,7 @@ zip_thumbnail (GsfInfile *infile, const char *out_filename, int thumb_size)
 		write_thumbnail (out_filename, data, len, thumb_size);
 		g_object_unref (thumbnail);
 	/* Check MS Office Open thumbnail */
-	} else if (NULL != (thumbnail = gsf_open_pkg_open_rel_by_type (GSF_INPUT(infile), 
+	} else if (NULL != (thumbnail = gsf_open_pkg_open_rel_by_type (GSF_INPUT(infile),
 				"http://schemas.openxmlformats.org/package/2006/relationships/metadata/thumbnail",
 																   NULL))) {
 		gsf_off_t len = gsf_input_remaining (thumbnail);
@@ -250,7 +253,7 @@ read_thumbnail_and_write (const char *in_filename, const char *out_filename, int
 static void
 set_resource_limits (void)
 {
-#ifdef HAVE_SETRLIMI
+#ifdef HAVE_SETRLIMIT
 	struct rlimit limit;
 
 	/* We call convert(1) from ImageMagick, which is especially scary when converting
